
//////////////////////////////////////////////////
//  PostgreSQL Data Access Components
//  Copyright  2008-2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I PgDac.inc}

unit PgConnectionEditor;
{$ENDIF}
interface
uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, Buttons, 
  {$IFDEF FPC}MaskEdit,{$ELSE}Mask,{$ENDIF}
{$IFDEF FPC}
  LResources,
{$ENDIF}
  CRTypes, DBAccess, PgAccess,
  DAConnectionEditor, TypInfo, CREditor;

type
  TPgConnectionEditorForm = class(TDAConnectionEditorForm)
    lbDatabase: TLabel;
    edDatabase: TComboBox;
    lbPort: TLabel;
    edPort: TEdit;
    edSchema: TEdit;
    lbSchema: TLabel;
    procedure edPortExit(Sender: TObject);
    procedure edPortChange(Sender: TObject);
    procedure edDatabaseDropDown(Sender: TObject);
    procedure edDatabaseChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure edSchemaChange(Sender: TObject);
    procedure edSchemaExit(Sender: TObject);
  protected
    function GetConnection: TPgConnection;
    function GetLocalConnection: TPgConnection;

    procedure DoInit; override;
    procedure DoSaveConnection; override;
    procedure FillInfo(InfoConnection: TCustomDAConnection); override;
    procedure ConnToControls; override;
    procedure AssignPort(const Value: Integer);
    procedure AssignDataBase(const Value: string);
    procedure AssignSchema(const Value: string);

  public
  {$IFNDEF CLR}
    procedure ReplaceEdit(var Edit: TWinControl);
    procedure SpinEditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  {$ENDIF}
    property Connection: TPgConnection read GetConnection;
    property LocalConnection: TPgConnection read GetLocalConnection;
end;

implementation

{$IFNDEF FPC}
{$IFDEF IDE}
{$R *.dfm}
{$ELSE}
{$R PgConnectionEditor.dfm}
{$ENDIF}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CRFunctions, DAConsts, PgCall, 
  DacVcl, PgDacVcl, PgConsts;

procedure TPgConnectionEditorForm.DoInit;
{$IFDEF MSWINDOWS}
{$IFNDEF CLR}
var
  WinControl: TWinControl;
{$ENDIF}
{$ENDIF}
begin
{$IFDEF MSWINDOWS}
{$IFNDEF CLR}
  WinControl := edPort;
  ReplaceEdit(WinControl);
{$ENDIF}
{$ENDIF}

  inherited;

  lbVersion.Caption := PgDACVersion +' ';
  UpdateVersionPosition;

{$IFDEF BETA}
  lbEdition.Caption := 'Beta';
  lbEdition.Font.Color := clRed;
{$ELSE}
{$IFDEF RC}
  lbEdition.Caption := 'Release Candidate';
  lbEdition.Font.Color := clGreen;
{$ELSE}
{$IFDEF TRIAL}
  lbEdition.Caption := 'Trial Edition';
{$ELSE} //ORIGINAL
{$IFDEF STD}
  lbEdition.Caption := 'Standard Edition';
{$ELSE}
  lbEdition.Caption := 'Professional Edition';
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}
end;

procedure TPgConnectionEditorForm.DoSaveConnection;
begin
  inherited;

  Connection.Database := LocalConnection.Database;
  Connection.Port := LocalConnection.Port;
  Connection.Schema := LocalConnection.Schema;
end;

procedure TPgConnectionEditorForm.FillInfo(InfoConnection: TCustomDAConnection);
var
  PgInfoConnection: TPgConnection;
begin
  PgInfoConnection := TPgConnection(InfoConnection);

  if not PgInfoConnection.Connected then
    meInfo.Lines.Clear
  else begin
    meInfo.Lines.BeginUpdate;
    try
      meInfo.Lines.Clear;
      meInfo.Lines.Add('PostgreSQL server version: ' + PgInfoConnection.ServerVersionFull);
      meInfo.Lines.Add('');
      meInfo.Lines.Add('PostgreSQL client version: Direct');
    finally
      meInfo.Lines.EndUpdate;
    end;
  end;
end;

procedure TPgConnectionEditorForm.ConnToControls;
begin
  inherited;

  edDatabase.Text := LocalConnection.Database;
  edPort.Text := IntToStr(LocalConnection.Port);
  edSchema.Text :=  LocalConnection.Schema;
end;

procedure TPgConnectionEditorForm.AssignPort(const Value: Integer);
begin
  if LocalConnection.Port <> Value then begin
    PerformDisconnect;
    LocalConnection.Port := Value;
    edPort.Text := IntToStr(Value);
    Modified := True;
  end;
end;

procedure TPgConnectionEditorForm.AssignDataBase(const Value: string);
begin
  if LocalConnection.Database <> Value then begin
    PerformDisconnect;
    LocalConnection.Database := Value;
    Modified := True;
  end;
end;

function TPgConnectionEditorForm.GetConnection: TPgConnection;
begin
  Result := FConnection as TPgConnection;
end;

function TPgConnectionEditorForm.GetLocalConnection: TPgConnection;
begin
  Result := FLocalConnection as TPgConnection;
end;

procedure TPgConnectionEditorForm.edPortExit(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  try
    try
      if edPort.Text = '' then
        AssignPort(PgDefValPort)
      else
        AssignPort(StrToInt(edPort.Text));
    except
      PageControl.ActivePage := shConnect;
      edPort.SetFocus;
      edPort.SelectAll;
      raise;
    end;
  finally
    ShowState;
  end;
end;

procedure TPgConnectionEditorForm.edPortChange(Sender: TObject);
begin
  if FInDoInit or (edPort.Text = '') then
    Exit;

  try
    try
      AssignPort(StrToInt(edPort.Text));
    except
      PageControl.ActivePage := shConnect;
      edPort.SetFocus;
      edPort.SelectAll;
      raise;
    end;
  finally
    ShowState;
  end;
end;

procedure TPgConnectionEditorForm.edDatabaseDropDown(Sender: TObject);
var
  OldConnected: boolean;
  OldDatabase: string;
  List: TStringList;
begin
{$IFDEF UNIX}
  edDatabase.OnGetItems := nil;
  try
{$ENDIF}
  StartWait;
  try
    TDBAccessUtils.SetLockLoginPrompt(LocalConnection, True);
    try
      OldConnected := LocalConnection.Connected;
      OldDatabase := LocalConnection.Database;
      if not OldConnected then
        LocalConnection.Database := 'template1';
      edDatabase.Items.Clear; // for case when GetDatabaseNames raises an exception
      List := TStringList.Create;
      try
      {$IFDEF DARWIN}
        try
      {$ENDIF}
          LocalConnection.GetDatabaseNames(List);
      {$IFDEF DARWIN}
        except
          on E:Exception do
            ShowMessage(E.Message);
        end;
      {$ENDIF}
        AssignStrings(List, edDatabase.Items);
      finally
        LocalConnection.Connected := OldConnected;
        if not OldConnected then
          LocalConnection.Database := OldDatabase;
        List.Free;
      end;
    finally
      TDBAccessUtils.SetLockLoginPrompt(LocalConnection, False);
    end;

    if edDatabase.Items.Count < 20 then
      edDatabase.DropDownCount := edDatabase.Items.Count
    else
      edDatabase.DropDownCount := 20;
  finally
    StopWait;
  end;
{$IFDEF UNIX}
  finally
    edDatabase.OnGetItems := edDatabaseDropDown;
  end;
{$ENDIF}
end;

{$IFNDEF CLR}
procedure TPgConnectionEditorForm.ReplaceEdit(var Edit: TWinControl);
type
  TSetProc = procedure (Self: TObject; Ptr: pointer);
var
  EditClass: string;
  NewEdit: TWinControl;
  OldName: string;
  TypeInfo: PTypeInfo;
begin
  if GetClass('TSpinEdit') <> nil then
    EditClass := 'TSpinEdit'
  else
{$IFDEF BCB}
  if GetClass('TCSpinEdit') <> nil then
    EditClass := 'TCSpinEdit'
  else
{$ENDIF}
    EditClass := '';

  if EditClass <> '' then begin
    NewEdit := TCustomControl(GetClass(EditClass).NewInstance);
    NewEdit.Create(Edit.Owner);

    with NewEdit do begin
      Parent := Edit.Parent;
      Left := Edit.Left;
      Top := Edit.Top;
      Width := Edit.Width;
      Height := Edit.Height;
      Align := Edit.Align;
      TabOrder := Edit.TabOrder;
      Anchors := Edit.Anchors;
      //Constraints := Edit.Constraints;
      TypeInfo := GetClass(EditClass).ClassInfo;
      HelpContext := Edit.HelpContext;

      if Edit is TEdit then begin
        SetReadOnly(NewEdit, TEdit(Edit).ReadOnly);
        SetOrdProp(NewEdit, 'Color', Longint(TEdit(Edit).Color));
      end;

      OnKeyDown := SpinEditKeyDown;
      if GetPropInfo(Edit.ClassInfo, 'OnChange') <> nil then
        SetMethodProp(NewEdit, GetPropInfo(TypeInfo, 'OnChange'),
          GetMethodProp(Edit, GetPropInfo(Edit.ClassInfo, 'OnChange')));
      SetMethodProp(NewEdit, GetPropInfo(TypeInfo, 'OnExit'),
        GetMethodProp(Edit, GetPropInfo(Edit.ClassInfo, 'OnExit')));
      SetMethodProp(NewEdit, GetPropInfo(TypeInfo, 'OnKeyDown'),
        GetMethodProp(Edit, GetPropInfo(Edit.ClassInfo, 'OnKeyDown')));
      SetMethodProp(NewEdit, GetPropInfo(TypeInfo, 'OnKeyPress'),
        GetMethodProp(Edit, GetPropInfo(Edit.ClassInfo, 'OnKeyPress')));
    end;

    if (Edit.Owner <> nil) and (TForm(Edit.Owner).ActiveControl = Edit) then
      TForm(Edit.Owner).ActiveControl := NewEdit;

    OldName := Edit.Name;
    Edit.Free;
    Edit := TEdit(NewEdit);
    NewEdit.Name := OldName;

    if (EditClass = 'TSpinEdit') {$IFDEF BCB} or (EditClass = 'TCSpinEdit' ){$ENDIF} then begin
      SetOrdProp(NewEdit, 'MaxValue', 65535);
      SetOrdProp(NewEdit, 'MinValue', 0);
    end;
  end;
end;

procedure TPgConnectionEditorForm.SpinEditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = 13 then
    btConnectClick(self);
end;
{$ENDIF}

procedure TPgConnectionEditorForm.edDatabaseChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  try
    AssignDataBase(edDatabase.Text);
  finally
    ShowState;
  end;
end;

procedure TPgConnectionEditorForm.FormShow(Sender: TObject);
begin
  inherited;
  TDBAccessUtils.SetLockLoginPrompt(LocalConnection, True);
end;

procedure TPgConnectionEditorForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  inherited;
  if CanClose then
    TDBAccessUtils.SetLockLoginPrompt(LocalConnection, False);
end;

procedure TPgConnectionEditorForm.edSchemaChange(Sender: TObject);
begin
  if FInDoInit or (edSchema.Text = '') then
    Exit;

  try
    try
      AssignSchema(edSchema.Text);
    except
      PageControl.ActivePage := shConnect;
      edSchema.SetFocus;
      edSchema.SelectAll;
      raise;
    end;
  finally
    ShowState;
  end;

end;

procedure TPgConnectionEditorForm.AssignSchema(const Value: string);
begin
  if LocalConnection.Schema <> Value then begin
    LocalConnection.Schema := Value;
    edSchema.Text := Value;
    Modified := True;
  end;
end;

procedure TPgConnectionEditorForm.edSchemaExit(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  try
    try
      AssignSchema(edSchema.Text);
    except
      PageControl.ActivePage := shConnect;
      edSchema.SetFocus;
      edSchema.SelectAll;
      raise;
    end;
  finally
    ShowState;
  end;


end;

end.
